package ch.unizh.ini.caviar.retina;
/*
 * CamRead.java
 *
 * Created on February 10, 2004, 2:32 PM
 */
import java.awt.*;
import java.awt.image.*;
import java.util.Random;
import java.util.prefs.*;
import javax.swing.*;
import java.io.*;

/**
 *
 * @author  tvst
 */
public class CamShow extends java.awt.Frame {
    
    Preferences prefs=Preferences.userNodeForPackage(CamShow.class);
    
    int imgWidth, imgHeight, dim;
    int minWidth= 200, minHeight= 200;
    boolean stopped;
    boolean loggingEnabled=false;
    CommThread visualization;
    final int BORDER_WIDTH=40;
    
    
    /** Creates new form CamRead */
    public CamShow() {
        stopped= true;
        
        initComponents();
        colorButtonGroup.add(useColorRadioButton);
        colorButtonGroup.add(useBWRadioButton);
        
        widthField.setText(Integer.toString(CamConstants.getColumns()));
        heightField.setText(Integer.toString(CamConstants.getRows()));
        
        imgWidth = Integer.parseInt(this.widthField.getText());
        imgHeight = Integer.parseInt(this.heightField.getText());
        dim = imgWidth * imgHeight;
        
        int z=prefs.getInt("zoom", 3);
        zoomChooser.setSelectedIndex(z);
        
        samplesBox.setText(Integer.toString(prefs.getInt("samples",100)));
        
        this.setLocation(300, 0); // location of visualization frame
        updateSize(CamConstants.getColumns(), CamConstants.getRows());
        setVisible(true);
        
        this.toolBox.pack();
        this.toolBox.setVisible(true);
        
        visualizationPanel.createBufferStrategy(2);
        startStop();
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new CamShow();
    }
    
    private void updateSize() {
        updateSize(imgWidth, imgHeight);
    }
    
    private void updateSize(int w, int h) {
        this.visualizationPanel.setSize(w, h);  // set the image size in screen pixels. this sets the java component size
        this.setSize(w + BORDER_WIDTH*2, h + BORDER_WIDTH*2); // set this Frame a little bigger
        this.visualizationPanel.setLocation(BORDER_WIDTH, BORDER_WIDTH); // put the retina image in the middle
        
    }
    
    private void zoom() {
        int z=zoomChooser.getSelectedIndex();
        prefs.putInt("zoom",z);
        double level= Math.pow(2, z); // level is 1,2,4,8, etc
        
        int w= (int) (imgWidth * level);
        int h= (int) (imgWidth * level);
        
        updateSize(w, h); // zoom up screen size of retina image
        
        if (this.visualization != null) {
            this.visualization.zoom(level); // tell the retina image to zoom
            this.visualization.updateImage(); // and update it
        }
        //        repaint();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        toolBox = new javax.swing.JFrame();
        zoomLabel = new javax.swing.JLabel();
        widthLabel = new javax.swing.JLabel();
        heightLabel = new javax.swing.JLabel();
        widthField = new javax.swing.JTextField();
        heightField = new javax.swing.JTextField();
        zoomChooser = new javax.swing.JComboBox();
        samplesBox = new javax.swing.JTextField();
        refreshLabel = new javax.swing.JLabel();
        samplesLabel = new javax.swing.JLabel();
        loggingToggleButton = new javax.swing.JToggleButton();
        startStop = new javax.swing.JButton();
        colorPanel = new javax.swing.JPanel();
        useBWRadioButton = new javax.swing.JRadioButton();
        useColorRadioButton = new javax.swing.JRadioButton();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        colorButtonGroup = new javax.swing.ButtonGroup();
        visualizationPanel = new java.awt.Canvas();

        toolBox.getContentPane().setLayout(new java.awt.GridBagLayout());

        toolBox.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        toolBox.setTitle("Options");
        toolBox.setBackground(java.awt.SystemColor.control);
        toolBox.setName("toolBox");
        toolBox.setResizable(false);
        zoomLabel.setText("Zoom:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.ipady = 4;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        toolBox.getContentPane().add(zoomLabel, gridBagConstraints);

        widthLabel.setText("Width:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.ipady = 4;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 44);
        toolBox.getContentPane().add(widthLabel, gridBagConstraints);

        heightLabel.setText("Height:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 1;
        gridBagConstraints.ipady = 4;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 42);
        toolBox.getContentPane().add(heightLabel, gridBagConstraints);

        widthField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        widthField.setText("64");
        widthField.setToolTipText("Retina width");
        widthField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                widthFieldActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 22;
        gridBagConstraints.insets = new java.awt.Insets(0, 5, 0, 0);
        toolBox.getContentPane().add(widthField, gridBagConstraints);

        heightField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        heightField.setText("64");
        heightField.setToolTipText("Retina height");
        heightField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                heightFieldActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipadx = 22;
        gridBagConstraints.insets = new java.awt.Insets(0, 50, 0, 2);
        toolBox.getContentPane().add(heightField, gridBagConstraints);

        zoomChooser.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1", "2", "4", "8", "16", "32", "64" }));
        zoomChooser.setSelectedIndex(5);
        zoomChooser.setToolTipText("Set the zoom level");
        zoomChooser.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                zoomChooserItemStateChanged(evt);
            }
        });
        zoomChooser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomChooserActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.ipadx = 88;
        gridBagConstraints.ipady = -5;
        gridBagConstraints.insets = new java.awt.Insets(10, 5, 0, 2);
        toolBox.getContentPane().add(zoomChooser, gridBagConstraints);

        samplesBox.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        samplesBox.setText("225");
        samplesBox.setToolTipText("Refresh every this many AER events");
        samplesBox.setEnabled(false);
        samplesBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                samplesBoxActionPerformed(evt);
            }
        });
        samplesBox.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                samplesBoxPropertyChange(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.ipadx = 18;
        gridBagConstraints.insets = new java.awt.Insets(10, 25, 33, 62);
        toolBox.getContentPane().add(samplesBox, gridBagConstraints);

        refreshLabel.setText("Refresh:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 32);
        toolBox.getContentPane().add(refreshLabel, gridBagConstraints);

        samplesLabel.setText("events");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(10, 40, 0, 3);
        toolBox.getContentPane().add(samplesLabel, gridBagConstraints);

        loggingToggleButton.setText("Enable logging");
        loggingToggleButton.setToolTipText("Enables AE event logging");
        loggingToggleButton.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        toolBox.getContentPane().add(loggingToggleButton, gridBagConstraints);

        startStop.setText("Start");
        startStop.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        startStop.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                startStopMouseClicked(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.ipadx = 136;
        gridBagConstraints.ipady = -2;
        gridBagConstraints.insets = new java.awt.Insets(14, 0, 3, 2);
        toolBox.getContentPane().add(startStop, gridBagConstraints);

        colorPanel.setBorder(new javax.swing.border.EtchedBorder());
        useBWRadioButton.setText("Gray");
        colorPanel.add(useBWRadioButton);

        useColorRadioButton.setText("Color");
        colorPanel.add(useColorRadioButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 3;
        toolBox.getContentPane().add(colorPanel, gridBagConstraints);

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        exitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, 0));
        exitMenuItem.setMnemonic('x');
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);

        toolBox.setJMenuBar(jMenuBar1);

        setLayout(null);

        setBackground(java.awt.SystemColor.control);
        setResizable(false);
        setTitle("CamRead");
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        visualizationPanel.setBackground(new java.awt.Color(0, 0, 0));
        add(visualizationPanel);
        visualizationPanel.setBounds(0, 0, 0, 0);

        pack();
    }//GEN-END:initComponents
    
    private void samplesBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_samplesBoxActionPerformed
        if (samplesBox.getText() == null || samplesBox.getText() == "" || Integer.parseInt(samplesBox.getText()) <= 0)
            JOptionPane.showMessageDialog(this, "Please enter a positive value.");
        prefs.putInt("samples",Integer.parseInt(samplesBox.getText()));
    }//GEN-LAST:event_samplesBoxActionPerformed
    
    private void heightFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_heightFieldActionPerformed
        CamConstants.setRows(Integer.parseInt(this.heightField.getText()));
    }//GEN-LAST:event_heightFieldActionPerformed
    
    private void widthFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_widthFieldActionPerformed
        CamConstants.setColumns(Integer.parseInt(this.widthField.getText()));
    }//GEN-LAST:event_widthFieldActionPerformed
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        System.exit(0);        // Add your handling code here:
    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    private void zoomChooserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomChooserActionPerformed
        zoom();
    }//GEN-LAST:event_zoomChooserActionPerformed
    
    private void samplesBoxPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_samplesBoxPropertyChange
        if (samplesBox.getText() == null || samplesBox.getText() == "" || Integer.parseInt(samplesBox.getText()) <= 0)
            JOptionPane.showMessageDialog(this, "Please enter a positive value.");
        prefs.putInt("samples",Integer.parseInt(samplesBox.getText()));
    }//GEN-LAST:event_samplesBoxPropertyChange
    
    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        this.toolBox.setVisible(true);
    }//GEN-LAST:event_formMouseClicked
    
    private void zoomChooserItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_zoomChooserItemStateChanged
        zoom();
    }//GEN-LAST:event_zoomChooserItemStateChanged
    
    private void startStopMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startStopMouseClicked
        startStop();
    }//GEN-LAST:event_startStopMouseClicked
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    
    void startStop() {
        if (stopped) { // start
            this.startStop.setText("Stop");
            this.samplesBox.setEnabled(false);
            this.heightField.setEnabled(false);
            this.widthField.setEnabled(false);
            stopped= false;
            
            try {
                
                imgWidth= CamConstants.getColumns();
                imgHeight= CamConstants.getRows();
                updateSize();
                
                visualization= new CommThread(  this.visualizationPanel,
                        imgWidth,
                        imgHeight,
                        Integer.parseInt(this.samplesBox.getText())
                        );
            } catch (Exception e) {
                System.out.println(e.toString());
            }
            
            zoom();
            visualization.start();
        } else { // stop
            this.startStop.setText("Start");
            this.samplesBox.setEnabled(true);
            this.heightField.setEnabled(true);
            this.widthField.setEnabled(true);
            stopped= true;
            visualization.stopExecution();
            //            visualization= null;
        }
    }
    
//    Random rand=new Random();
    
    public void paint(Graphics graphics) {
        if (this.visualization != null) {

            // draw black
            int z=zoomChooser.getSelectedIndex();
            double level= Math.pow(2, z); // level is 1,2,4,8, etc
            int w= (int) (imgWidth * level);
            int h= (int) (imgWidth * level);
 
            this.visualization.g.setColor(Color.black);
            this.visualization.g.fillRect(0,0, w,h);
            
            // draw image
            this.visualization.g.drawImage(this.visualization.img , 0, 0, this);
        }
    }
    
 // not overriding update calls paint directly 
//    public void update(Graphics graphics) {
//        //        System.out.println("                                           update graphics");
//        paint(graphics);
//    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup colorButtonGroup;
    private javax.swing.JPanel colorPanel;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JTextField heightField;
    private javax.swing.JLabel heightLabel;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JToggleButton loggingToggleButton;
    private javax.swing.JLabel refreshLabel;
    private javax.swing.JTextField samplesBox;
    private javax.swing.JLabel samplesLabel;
    private javax.swing.JButton startStop;
    private javax.swing.JFrame toolBox;
    private javax.swing.JRadioButton useBWRadioButton;
    private javax.swing.JRadioButton useColorRadioButton;
    private java.awt.Canvas visualizationPanel;
    private javax.swing.JTextField widthField;
    private javax.swing.JLabel widthLabel;
    private javax.swing.JComboBox zoomChooser;
    private javax.swing.JLabel zoomLabel;
    // End of variables declaration//GEN-END:variables
    
}