package ch.unizh.ini.caviar.retina;

import ch.unizh.ini.caviar.usbaemonitor.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.*;

/** receives AE from retina and paints the images */
public class CommThread extends Thread {
    
    public Image img; // this is actual image that is painted
    public MemoryImageSource imgsrc; // this is memory image source that is default ARGB model that is written to to make the displayed image
    public Component display, topFrame;
    boolean stop;//, receivedAtLeastOnePacket;
    byte oneComponent;
    public int[] pixels, histogram; // pixels are created from the histogram of counted events. imagesrc is created from these pixels.
    int imgWidth, imgHeight, zoomWidth, zoomHeight, dim, eventCounter, newPixel;
    int maxPixel=Integer.MIN_VALUE, minPixel=Integer.MAX_VALUE; // used for scaling
    private boolean autoscaleEnabled=true; // automatically scale to max/min of pixels
    private boolean colorEnabled=true; // set true to make green positive, red negative
    private int scale; // image brightness full scale, either automatic or manual
    public Graphics g;
    public double zoomLevel;
    double mark;
    Timer timer;
    int eventsPerRefresh, avg;
    long packetPeriod, framePeriod;
    USBAEMonitor reader;
    boolean readerOpen=false;
    VolatileImage volimg;
            
    /**
     *@param displayComponent Component in which to display image
     *@paraw w width of retina in pixels
     *@param h height of retina in retina pixels
     *@param s events per refresh
     */
    CommThread(Component displayComponent, int w, int h, int s) {
        display= displayComponent;
        topFrame= displayComponent.getParent();
        g= display.getGraphics();
        dim= w*h;
        imgWidth= w;
        imgHeight= h;
        eventsPerRefresh= s;
        zoomLevel= 1;
        zoomWidth= zoomHeight= 0;
        pixels= new int[dim];
        histogram= new int[dim];
//        timer= new Timer();
        
        imgsrc= new MemoryImageSource(imgWidth, imgHeight, pixels, 0, imgWidth);
        imgsrc.setAnimated(true);
        imgsrc.setFullBufferUpdates(true);
        img= (zoomLevel == 1)? display.createImage(imgsrc): display.createImage(imgsrc).getScaledInstance(zoomWidth, zoomHeight, Image.SCALE_FAST);        
        img.setAccelerationPriority(1f);

        reader = new USBAEMonitor();
        System.out.println("Registering shutdown hook to close USBAEMonitor");
        Runtime.getRuntime().addShutdownHook(new Thread(){
            public void run(){
                if(reader!=null){
                    reader.close();
                }
            }
        });
    }
    
    /** zooms image. really not zoom but sizing factor. you don't zoom up on part of the retina, you blow up the size of
     *the entire image.
     *@param level zoom factor, 1 means no zoom: retina pixel=screen pixel
     */
    public void zoom(double level) {
        zoomLevel= level;
        zoomWidth= (int) (imgWidth * level);
        zoomHeight= (int) (imgHeight * level);
        img= (zoomLevel == 1)? display.createImage(imgsrc): display.createImage(imgsrc).getScaledInstance(zoomWidth, zoomHeight, Image.SCALE_FAST);
   }
    
    public void stopExecution() {
        stop = true;
    }
    
    public void clearImage() {
        clearVariables();
        updateImage();
    }
    
    
    /** makes a new MemoryImageSource using the pixels, then creates an image with that imagesource.
     * ( or with a scaled copy). called by display(). */
    public void updateImage() {
//        System.out.println("updateImage");
//        imgsrc= new MemoryImageSource(imgWidth, imgHeight, pixels, 0, imgWidth);
//        imgsrc= new MemoryImageSource(imgWidth, imgHeight, pixels, 0, imgWidth);
//        img= (zoomLevel == 1)? display.createImage(imgsrc): display.createImage(imgsrc).getScaledInstance(zoomWidth, zoomHeight, Image.SCALE_FAST);
          imgsrc.newPixels(0,0,imgWidth,imgHeight,true);
  //      display.repaint(20); // call for a repaint within some time
    }
    
    void computeAutoScale(){
        int s=Math.max(maxPixel,-minPixel);
        setScale(s);
    }
    /** uses the image histogram to set the pixel colors. called from the thread run loop when enough events have accumulated. */
    void display() {
        oneComponent=0; // one color component
        
//        System.out.println("set pixels");
        float s=getScale();
        if(isColorEnabled()){
            for (int i=0; i < dim; i++) {
                int p=histogram[i];
                if(p>0){ //green
                    oneComponent= (byte) ((histogram[i] / (s)) * 0xFF);
                    pixels[i]= 0xFF000000 |  ((oneComponent << 8) & 0x0000FF00) ;
                }else if(p<0){ //red
                    oneComponent= (byte) ((histogram[i] / (s)) * 0xFF);
                    pixels[i]= 0xFF000000 | ((oneComponent << 16) & 0xFF0000) ;
                }else{
                    pixels[i]=0;
                }
            }
        }else{
            for (int i=0; i < dim; i++) {
                oneComponent= (byte) ((histogram[i] / (s)) * 255);
                pixels[i]= 0xFF000000 | ((oneComponent << 16) & 0xFF0000) | ((oneComponent << 8) & 0x00FF00) | (oneComponent & 0x0000FF);
            }
        }
        updateImage();
        clearVariables();
    }
    
    void clearVariables() {
        // clear variables
//        System.out.println("clear vars");
        if(!colorEnabled){
            for(int i=0;i<dim;i++){
                histogram[i]=0x80;
            }
        }else{
            for(int i=0;i<dim;i++){
                histogram[i]=0x00;
            }
        }
        maxPixel=Integer.MIN_VALUE;
        minPixel=Integer.MAX_VALUE;
    }
    
    /** run method. loops until stop is true, acquiring events into a histogram.
     * Actual painting of the histogram is done in the graphics context.
     *Histogram is only rendered into frame when sufficient events have been accumulated.
     */
    public void run() {
        
        try{
            reader.open();
            stop=false;
        }catch(USBAEMonitorException e){
            e.printStackTrace();
            stop=true;
        }
        
        setPriority(Thread.MIN_PRIORITY);
        

        // update interface
        clearImage();
        eventCounter= 0;
        // loop until we're told to stop
        while (!stop) {
            try {
                Thread.currentThread().sleep(40); // this affects flicker because we're not double buffered
                int numEvents=reader.acquireAvailableEventsFromDriver();
            } catch (Exception e) {
                e.printStackTrace();
                return;
            }
            
            // decode packet
            int numEvents=reader.getNumEventsAcquired();
            //            if(numEvents==0) System.out.println("no events");
            if(numEvents>0){
                //                System.out.println(numEvents+" events");
                short[] addr=reader.getAddresses();
                //int [] ts=reader.getTimestamps();
                int newPixelValue;
                for(int j=0;j<numEvents;j++){
                    short a=addr[j];
                    short x=(short)((a&0x7e)>>1); // map to retina address space
                    short y=(short)((a&0x3f00)>>8);
                    byte pol=(byte)(((int)a&1)==1?1:-1);
                    newPixel=x+y*imgWidth;
                    newPixelValue=(histogram[newPixel]+=pol);
                    if(newPixelValue>maxPixel) maxPixel=newPixelValue;
                    if(newPixelValue<minPixel) minPixel=newPixelValue;
                }
                eventCounter+=numEvents;
                 display();
           }
            //            System.out.println("eventCounter="+eventCounter);
            //            if (eventCounter >= eventsPerRefresh) {
            
            eventCounter = 0;
            //            }
        }
    }
    
    public boolean isColorEnabled() {
        return colorEnabled;
    }
    
    public void setColorEnabled(boolean colorEnabled) {
        this.colorEnabled = colorEnabled;
    }
    
    public int getScale() {
        if (autoscaleEnabled){
            computeAutoScale();
        }
        return scale;
    }
    
    public void setScale(int scale) {
        this.scale = scale;
    }
    
    public boolean isAutoscaleEnabled() {
        return autoscaleEnabled;
    }
    
    public void setAutoscaleEnabled(boolean autoscaleEnabled) {
        this.autoscaleEnabled = autoscaleEnabled;
    }
}

