/* 
mex source for USB AE Monitor using simple Sevilla USB AE board and SiLabs USBXPress
	this should be built using matlab's lcc included C compiler.
	mex -setup
		and choose microsoft visual C++ compiler
	mex -g usbaemon.cpp SiUSBXp.lib

  NOTE: file extension MUST BE .cpp and not .c in order to implicitly link to the SI DLL functions!
	
 -tobi feb 2005
 */

#include <windows.h>	// include this to define DWORD, etc.
#include "mex.h"
#include "SiUSBXp.h"	// customized version of SILabs header modified for C from C++

#define DEVICE_STILL_THERE_POLLING_INTERVAL_MS 500 // time in ms between device still plugged in polling
#define DRIVER_REOPEN_NO_EVENTS_TIMEOUT_MS 5000 // time in ms between closing/reopening driver when no events come
#define ERROR_PRINT_INTERVAL_MS 10000 // time in ms between printing error/warning messages
//#define SLEEP_WAIT_MS	3	// defines sleep time in ms during polling of USB driver for data
#define TIMEOUT_MS		25	// defines timeout for read, write. read will return with available events in this time
							// even if all are not available
#define TICK_US	1			// microcontroller event timer tick in us

// MS VC should compile this ".c" file with the plain C compiler based on ".c" extension, so no extern "C" is needed

// these are declared static to be invisible outside
static void closeDevice(void); // exit handler
static void printErrorStatus(SI_STATUS);
static void flushBuffers(void);
// deviceHandle is static local var so that mex exit handler can use it to close device	
static HANDLE deviceHandle;	// handle returned to device by SI_Open. used by SI_Close and mexAtExit/CloseStream


/*
// checks for user control-C during poll of driver for events
bool checkUserBreak(void){
	MSG msg;
	BOOL bRet;

	bRet = GetMessage(&msg, (HWND) NULL, 0, 0);
	if (bRet == -1){
		mexErrMsgTxt("error in GetMessage");
		return true;
	}else{ // got message successfully
		if(TranslateMessage(&msg)!=0){ // if 0 we got character message
			switch (msg.wParam) 
			{ 
				case 0x03: 
            
					// Process a control-C. 
             
					return true; // interrupt
				 default: 
					// anything else, no interrupt
					return false; 
			} 
		}
	}
	return false;
}
*/

// main mex function
// has exactly one function: to return available events
// if DLL is not loaded it will be
// if device is not opened, it will be
// returns 2 vectors, [addr,shortts]. 
// addr are uint16 addresse
// shortts are uint32 timestamps with 1us tick. these are unwrapped here in mex file from 16 bit 2us-tick timestamps.
void
mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{

 	unsigned int j,k,n;
 	int numEvents,eventCounter;
// 	int bytes_to_copy;
 	unsigned short addr,*aePtr;
	unsigned short shortts;	// raw 16 bit timestamp from device
	unsigned int *tsPtr;	// pointer to 32 bit returned timetamp arrar
	//int readyToRead;	// for rcv queue
	bool overrunOccured=false;

	// static so persistent
	static DWORD deviceNum;	//	usb device number (will almost always be 0)
	//static HMODULE lib;	// DLL SiUSBXp.dll
	static unsigned char aeBuffer[SI_MAX_READ_SIZE]; // static buffer for collected events
	static unsigned int wrapAdd=0;		// static offset to add to spike timestamps
	static unsigned short lastshortts=0;		// holds last 16 bit timestamp of previous event, to detect wrapping
	static DWORD lastTimeCheckedDeviceThere;	// used from periodic polling to detect unplugging of device
	static DWORD lastTimeGotEvent; // used to close and reopen periodically to refresh driver, which tends to go stale if nothing comes over the channel
	static DWORD lastTimeErrorPrinted;

	DWORD	numDevices;
	SI_STATUS status;
	char deviceString[SI_MAX_DEVICE_STRLEN], serialNumberString[SI_MAX_DEVICE_STRLEN];    
	DWORD bytesRead;
	DWORD bytesInQueue;
	DWORD queueStatus; // ,bytesWritten;
	
    /* Examine input (right-hand-side) arguments. */
    if(nrhs!=0){
    	mexErrMsgTxt("doesn't take any arguments");
	}
    
    if(nlhs>3){
    	mexErrMsgTxt("[addr,shortts,overrun]=usbaemon. \naddr are uint16 addresses\nts are uint32 timestamps with 1us tick that are unwrapped here in mex file from 16 bit 2us-tick timestamps.\noverrun is set to 1 if buffer overun occured, otherwise 0.");
	}
    	
	//mexPrintf("starting usbaemon\n");
	
	// DLL is left resident and mex file is locked in memory to save static vars
	// we don't do this now and instead register exit function to close driver
	//mexLock();
	//mexPrintf("locked mex file\n");

	// start actual use of functions from library, finally!!!
	if(deviceHandle==NULL){ // we only open device once
		//mexPrintf("finding number of attached devices\n");
		status=(SI_GetNumDevices)(&numDevices);
		if(status!=SI_SUCCESS){
			mexPrintf("usbaemon: error SI_GetNumDevices, numDevices=%d, SI_STATUS=0x%x\n",numDevices,status);
			if(status==0xff){mexPrintf("SI_DEVICE_NOT_FOUND \n(USBXPress Device driver not installed, cable not connected, device not powered?)\n");}
			printErrorStatus(status);
			mexErrMsgTxt("usbaemon: error SI_GetNumDevices");
		}	
		if(numDevices==0){
			mexErrMsgTxt("error SI_GetNumDevices no devices");
		}else if(numDevices>1){
			mexPrintf("usbaemon: %d device(s) found, choosing first one\n",numDevices);
		}
		deviceNum=0;	// always choose first device
		

		deviceString[0]=0;
		status=(SI_GetProductString)(deviceNum,deviceString,SI_RETURN_DESCRIPTION);
		if(status!=SI_SUCCESS){
			printErrorStatus(status);
			mexErrMsgTxt("error SI_GetProductString(DeviceString)");
		}
		serialNumberString[0]=0;
		status=(SI_GetProductString)(deviceNum,serialNumberString,SI_RETURN_SERIAL_NUMBER);
		if(status!=SI_SUCCESS){
			printErrorStatus(status);
			mexErrMsgTxt("error SI_GetProductString(SerialNumber)");
		}
		mexPrintf("usbaemon: product string: %s, Serial Number: %s\n",deviceString,serialNumberString); // this never returns something sensible
			
		status = (SI_SetTimeouts)(TIMEOUT_MS,TIMEOUT_MS); // set readtimeout, writetimeout  for recovery from hang
															// write timeout set even though we never write (so far)
		if(status!=SI_SUCCESS){
			printErrorStatus(status);
			mexErrMsgTxt("error SI_SetTimeouts");
		}
		//mexPrintf("setTimeouts OK\n");

		status = (SI_Open)(0, &deviceHandle);
		if(status!=SI_SUCCESS){
			printErrorStatus(status);
			mexErrMsgTxt("error SI_Open");
		}
//		mexPrintf("device opened, device handle=%d\n",deviceHandle);
		
		/* Register an exit function. You should only register the
		   exit function after the device has been opened successfully*/
		mexAtExit(closeDevice);
//		mexPrintf("registered onExit function OK\n");

		//flushBuffers();
		wrapAdd=0;
		//mexPrintf("setTimeouts OK\n");

		// init timers
		lastTimeCheckedDeviceThere=GetTickCount();
		lastTimeGotEvent=GetTickCount();
		lastTimeErrorPrinted=GetTickCount();
		
	} // open device

    
    
    // write a message to stimulate tranmission of events
	// not used now, as long as device is open events are written

    //SI_STATUS SI_Write (HANDLE Handle, LPVOID Buffer, DWORD NumBytesToWrite,DWORD*NumBytesWritten);
/*    status=(SI_Write)(deviceHandle,&msg,1,&bytesWritten);
    if(status!=SI_SUCCESS){
    	mexPrintf("error writing message, status=%d\n",status);
		mexErrMsgTxt("error SI_Write");
	}
	mexPrintf("wrote msg to start xmit from device\n");	
*/
    	

/*
	// check rcv queue

	// RX Queue status flags #define SI_RX_NO_OVERRUN 0x00 #define SI_RX_OVERRUN 0x01 #define SI_RX_READY 0x02

	readyToRead=0;
	bytesInQueue=0;
	while(!readyToRead){
		status=(SI_CheckRXQueue)(deviceHandle,&bytesInQueue,&queueStatus);
		if(status!=SI_SUCCESS){
	 		mexPrintf("error SI_CheckRXQueue: status=%d, queueStatus=%d\n",status,queueStatus);
			printErrorStatus(status);
			mexErrMsgTxt("error SI_CheckRXQueue");
		}
		if(queueStatus&SI_RX_OVERRUN){
	 		mexPrintf("warning: SI_RX_OVERRUN, queueStatus=0x%x, %d bytes in queue, flushing buffers\n",queueStatus, bytesInQueue);
			flushBuffers();
 		}
 		if(queueStatus&SI_RX_READY){
 			readyToRead=1;
		}else{
//			if(readyToRead==0)mexPrintf("not ready to rx yet\n");
//
//			Sleep(SLEEP_WAIT_MS);
//			if(checkUserBreak()){
//				mexErrMsgTxt("user interrupt");
//			}
			mexCallMATLAB(0,0,0,0,"drawnow"); // to catch interrupts. method using windows hook is too obscure to figure out.
		}
	}
*/

	// every so often, check if device is still actually there
	// if there, just continue.
	// if error or no device found, return error and clear device handle. 
	// user code can try/catch around calls to maintain flexible connection
	if(GetTickCount()-lastTimeCheckedDeviceThere>DEVICE_STILL_THERE_POLLING_INTERVAL_MS){
		lastTimeCheckedDeviceThere=GetTickCount();
		status=(SI_GetNumDevices)(&numDevices);
		if(status!=SI_SUCCESS){
			closeDevice();  // next time we'll try opening again, with new handle
			mexErrMsgTxt("usbaemon: error SI_GetNumDevices");  // returns error
		}	
	}

	// we read all bytes available *if* driver says its ok to read anything
	// if driver isn't ready to read, we just return 0 events
	// if there is an overrun we detect it and return the events that have been stored
	// then we clear overrun with flushbuffers

	status=(SI_CheckRXQueue)(deviceHandle,&bytesInQueue,&queueStatus);
	if(status!=SI_SUCCESS){
	 	mexPrintf("usbaemon: error SI_CheckRXQueue: status=%d, queueStatus=%d\n",status,queueStatus);
		printErrorStatus(status);
		mexErrMsgTxt("error SI_CheckRXQueue");
	}
	if(queueStatus&SI_RX_OVERRUN){
	 	if(nlhs<3) // only print warning if user doesn't record overrun flag
			mexWarnMsgTxt("usbaemon: USBXPress driver event buffer overrun, can only buffer 16k events.");
		overrunOccured=true;
	}

	// read events
	//SI_STATUS SI_Read (HANDLE Handle, LPVOID Buffer, DWORD NumBytesToRead,DWORD*NumBytesReturned)
	bytesRead=0;
	// if there was overrun, bytesInQueue will be 0 even though there are 16k bytes available
	bytesInQueue=overrunOccured?SI_MAX_READ_SIZE:bytesInQueue; // read all if available, otherwise return all available within timeout
	status=(SI_Read)(deviceHandle,aeBuffer,bytesInQueue,&bytesRead);
	if(status!=SI_SUCCESS){
		if(status&SI_RX_QUEUE_NOT_READY) {
			bytesRead=0;
		}else{
			mexPrintf("usbaemon: error reading bytes: status=%d, bytesInQueue=%d\n",(int)status,(int)bytesInQueue);
			printErrorStatus(status);
			mexErrMsgTxt("usbaemon: error SI_Read");
		}
	}

	// if no events have been received for a long time, close and reopen device to refresh possiblly stale driver
	if(bytesInQueue>0){
		lastTimeGotEvent=GetTickCount();
	}else{
		if(GetTickCount()-lastTimeGotEvent>DRIVER_REOPEN_NO_EVENTS_TIMEOUT_MS){
			mexPrintf("usbaemon: No events for a while, reopening driver to refresh it\n");
			closeDevice();

			status=(SI_GetNumDevices)(&numDevices);
			if(status!=SI_SUCCESS){
				mexPrintf("error SI_GetNumDevices, numDevices=%d, SI_STATUS=0x%x\n",numDevices,status);
				if(status==0xff){mexPrintf("SI_DEVICE_NOT_FOUND \n(USBXPress Device driver not installed, cable not connected, device not powered?)\n");}
				printErrorStatus(status);
				mexErrMsgTxt("usbaemon: error SI_GetNumDevices");
			}	
			if(numDevices==0){
				mexErrMsgTxt("error SI_GetNumDevices no devices");
			}else if(numDevices>1){
				mexPrintf("usbaemon: %d device(s) found, choosing first one\n",numDevices);
			}
			deviceNum=0;	// always choose first device
							
			status = (SI_SetTimeouts)(TIMEOUT_MS,TIMEOUT_MS); // set readtimeout, writetimeout  for recovery from hang
																// write timeout set even though we never write (so far)
			if(status!=SI_SUCCESS){
				printErrorStatus(status);
				mexErrMsgTxt("error SI_SetTimeouts");
			}
			//mexPrintf("setTimeouts OK\n");

			status = (SI_Open)(0, &deviceHandle);
			if(status!=SI_SUCCESS){
				printErrorStatus(status);
				mexErrMsgTxt("error SI_Open");
			}
	//		mexPrintf("device opened, device handle=%d\n",deviceHandle);
			
			// init timers
			lastTimeCheckedDeviceThere=GetTickCount();
			lastTimeGotEvent=GetTickCount();
			
		}
	}


/*
// debug
	mexPrintf("%d bytes read\n",(int)bytesRead);
	n=bytesRead>8?8:bytesRead;
	for(j=0;j<n;j++){
		mexPrintf("%d,",(int)aeBuffer[j]);
	}
	mexPrintf(" ...\n");
//debug	
*/
	if (bytesRead%4!=0){
		mexPrintf("usbaemon: read %d bytes, which is not divisible by 4, something's wrong, closing\n", bytesRead);
		closeDevice();
		mexErrMsgTxt("usbaemon: error in reading events");
	}
	numEvents=bytesRead/4;

	if(overrunOccured) flushBuffers(); // clears overrun flag. we have read everything anyhow

//	mexPrintf("%d bytesRead, %d events\n",(int)bytesRead,(int)numEvents);
	
//		mexPrintf("creating output matrix 2 x %d\n",numEvents);
	if(nlhs>0){
		plhs[0] = mxCreateNumericMatrix(1,numEvents,mxUINT16_CLASS,mxREAL);
		aePtr = (unsigned short *)mxGetData(plhs[0]);
	}
	if(nlhs>1) {
		plhs[1]=mxCreateNumericMatrix(1,numEvents,mxUINT32_CLASS,mxREAL);
		tsPtr=(unsigned int *)mxGetData(plhs[1]);
	}
	if(nlhs>2) {
		plhs[2]=mxCreateDoubleScalar((double)overrunOccured);
	}


	/* Populate the real part of the created array.
	 ae and timestamps are sent from USB device in BIG-ENDIAN format. MSB comes first,
	The addresses are simply copied over, and the timestamps are unwrapped to make uint32 timestamps.
	wrapping is detected when the present timestamp is less than the previous one.
	then we assume the counter has wrapped--but only once--and add into this and subsequent
	timestamps the wrap value of 2^16. this offset is maintained and increases every time there 
	is a wrap. hence it integrates the wrap offset.
	the timestamp is returned in 1us ticks.
	this conversion is to make values more compatible with other CAVIAR software components.
	*/
	if(nlhs>0){
		eventCounter=0;
		for(j=0;j<bytesRead;j+=4){
			addr=aeBuffer[j+1]+((aeBuffer[j])<<8); // remember addr is sent big-endian
			aePtr[eventCounter]=addr;	// simply copy address
			if(nlhs>1) {	// if we're returning timestamps
				shortts=aeBuffer[j+3]+((aeBuffer[j+2])<<8); // this is 16 bit value of timestamp in 2us tick
				if(shortts<lastshortts){
					wrapAdd+=0x10000;	// if we wrapped then increment wrap value by 2^16
	//					mexPrintf("event %d wraps: lastshortts=0x%X, shortts=0x%X, wrapAdd now=0x%X, final ts/2=0x%X, final ts=0x%X\n",
	//						eventCounter+1,lastshortts,shortts,wrapAdd,(shortts+wrapAdd),(shortts+wrapAdd)*2);
				}
				tsPtr[eventCounter]=(shortts+wrapAdd)*TICK_US; //add in the wrap offset and convert to 1us tick
				lastshortts=shortts;	// save last timestamp
			}
			eventCounter++;
		}
	}
//		mexPrintf("%d events\n",eventCounter);

		
		/* from http://www.codeproject.com/cpp/endianness.asp
		Little-endian: Least significant byte is stored at the lowest byte address.
		The Intel x86 family and Digital Equipment Corporation architectures (PDP-11, VAX, Alpha) 
		are representatives of Little-Endian.

		The Keil compiler is big-endian, but events are devlivered into the pipe and are reassembled as shown above.

		*/
//		bytes_to_copy = bytesRead; // numEvents*2*mxGetElementSize(plhs[0]);
//		memcpy(aePtr, aeBuffer, bytes_to_copy);

		
}	// end of mexFunction

/* Here is the error function, which gets run when the MEX-file is
   cleared and when the user exits MATLAB. The mexAtExit function
   should always be declared as static. */
static void closeDevice(void)
{
  SI_STATUS status;
//  mexPrintf("calling SI_Close\n");
  if(deviceHandle!=NULL) {
		// SI_Close can hang if called after a long time of no events received
		// try to flush buffers according to forum article
		// this can also hang
		//status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers
		//if(status!=SI_SUCCESS){
			//mexPrintf("error SI_FlushBuffers, SI_STATUS=%d\n",status);
			//printErrorStatus(status);
		//}
  		status = (SI_Close)(deviceHandle);
		if(status!=SI_SUCCESS){
			mexPrintf("usbaemon: error SI_Close: status=%d\n",status);
			printErrorStatus(status);
			deviceHandle=NULL;
			mexErrMsgTxt("usbaemon: error SI_Close");
		}
	// mexPrintf("usbaemon: SI_Close successful\n");
	deviceHandle=NULL;
  }else{
	  mexErrMsgTxt("closeDevice: deviceHandle is NULL, can't close it");
  }
}

void printErrorStatus(SI_STATUS status){
	mexPrintf("usbaemon: Error ");
	switch(status){
		case 0x00:
			mexPrintf("0x00 SI_SUCCESS\n");
			break;
		case 0xFF:
			mexPrintf("0xFF SI_DEVICE_NOT_FOUND\n");
			break;
		case 0x01:
			mexPrintf("0x01 SI_INVALID_HANDLE\n");
			break;
		case 0x02:
			mexPrintf("0x02 SI_READ_ERROR\n");
			break;
		case 0x03:
			mexPrintf("0x03 SI_RX_QUEUE_NOT_READY\n");
			break;
		case 0x04:
			mexPrintf("0x04 SI_WRITE_ERROR\n");
			break;
		case 0x05:
			mexPrintf("0x05 SI_RESET_ERROR\n");
			break;
		case 0x06:
			mexPrintf("0x06 SI_INVALID_PARAMETER\n");
			break;
		case 0x07:
			mexPrintf("0x07 SI_INVALID_REQUEST_LENGTH\n");
			break;
		case 0x08:
			mexPrintf("0x08 SI_DEVICE_IO_FAILED\n");
			break;
		case 0x09:
			mexPrintf("0x09 SI_INVALID_BAUDRATE\n");
	}
}

void flushBuffers(){
	SI_STATUS status;
	status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers
	if(status!=SI_SUCCESS){
		mexPrintf("usbaemon: error SI_FlushBuffers, SI_STATUS=%d\n",status);
		printErrorStatus(status);
		mexErrMsgTxt("usbaemon: error SI_FlushBuffers");
	}
}

