% [addr,ts,overrun]=usbaemon
% (WINDOWS ONLY)
% collects address-event data from Sevilla mini USB AER board
% addr are uint16 addresses.
% ts are uint32 timestamps in microseconds, which have been unwrapped from uint16 values
% received from USB SI labs C8051F320 controller device.
% overrun is double flag that is set to 1 when there is a driver overrun
% (>16k events since last call) and 0 otherwise. If overrun is not an output parameter then warning
% messages are printed on driver overrun.
% 
% When invoked with no output arguments driver is opened and events start
% to be collected.
% If USBXPress DLL is not loaded, it will be.
% First call opens device, which remains open collecting events.
% To reset and reopen device, clear mex or clear usbaemon.
%
% The events are buffered in the SiLabs USB host driver, which
%    has a 64kB buffer, enough to hold 16kEvents. 
%
% The timestamps wrap at 32 bits*1us=4295 sec=71 minutes. They are
% reset when the device is opened. 
% 
% If there is a buffer overrun (>16k events since last read) 16k events are
% returned and you have probably lost the newest events.
% 
% To plot the addresses vs time:
% [addr,ts]=usbaemon; 
% plot(ts,addr);
% 
% If the cable is unplugged during operation it will be detected within 1
% second. Then the device can simply be polled in a try/catch loop and will
% recover when it is plugged in again, as follows:
% while 1, 
%     try
%         [a,t,o]=usbaemon; 
%     catch
%         drawnow; % to allow for interrupting with ^C (otherwise you can't stop!)
%         pause(1);
%         continue;
%     end
%   % process captured events here
% end
%
%
% The device buffer holds 900 bytes=900/4=225 events. Transfer of a full USB
% buffer requires about 860us, during which time the device does not
% acknowledge a request. Thus with high activity you will notice periodic jumps of about 850
% in the timestamps; these are the USB transfers.
%
% Handshake time from req active to ack inactive while acquiring events is measured to be about 4.3us.
% There are approx 860 us pauses every full buffer's worth of events for USB transfer.
% Therefore max theoretical xfer rate is about 225 events 
% in 4.7us*225events+860us = 225 events/1.8ms= 117k events/sec.
% This rate is reduced somewhat by transmitter's ability to 
% generate a new request after acknowledge goes inactive.
% Measured rate with tmpdiff11 retina (not tuned for max output rate) is
% about 100k events/sec. Peak rate is (during a frame) 117k events/sec
%
% When the device is not opened, then it just handshakes with AE
% transmitter, without storing events. Then the cycle time is about
% 1us/event.
%
% The LEDs indicate activity. 
% The middle LED indicates AE activity
% The LED closest to the USB connector indicates that the device has been USB opened.
% The LED farthest from the USB connector indicates USB transfers.
%
% usbaemon.dll needs to be on the matlab path to be recognized as a
% mex-function, and SiUSBLib.dll need to be somewhere on the Windows PATH.
% Otherwise you will get an error that the mex file can be loaded but not
% intialized.
%
% If the device is opened and read, and then there is a long period of no
% AE activity (say your device is turned off), then the driver has a bug
% that prevents returning any more events and causes a hang on close. It is
% necessary to unplug and replug the device to make it work again.
%
% see buildusbaemon testPollForEvents showLiveRetina testusbaemon
 
% author tobi jan-feb 2005

