function playRecordedObject(cmd)
% plays back events recorded with usbaemon
% prompts for .mat file holding allAddr and allTs recorded by logging from
% showLiveXXX
% type 'h' in figure for help

persistent filename frameDuration cs useColor usePolarity tstart tend
persistent stopEnabled pauseEnabled rewToStartEnabled rewEnabled ffEnabled
persistent backwardsEnabled rewindToStartEnabled
persistent playbackFigure timeSlider abststart abstend
persistent markStart markEnd loopEnabled renderer jetMap grayColorCs
persistent colorMapping im textEnabled numEvents sliderMoved recordMovieEnabled mov
persistent buttonDown sliderValue
persistent selectedPixel rotation

tick=1e-6; % timestamp tick in seconds for recorded data

d=16; % size of conv chip (this is not general, just for plot limits)

if nargin==0,
    [filename,path]=uigetfile('*.mat','Select recorded chip data file');
    if filename==0, return; end
    load([path,filename]);
    cd(path);
    if exist('ae','var') && size(ae,1)>0,
        ae = double (ae);
    elseif exist('allAddr','var') && exist('allTs','var'),
        ae=[double(allAddr);double(allTs)];
    else
        error 'data in matrix ae or vectors allAddr,allTs not found in this data file'
    end
    numEvents=size(ae,2);
    abststart=double(ae(2,1)); % first spike
    abstend=ae(2,end);
    fprintf('total duration %f seconds\n',(abstend-abststart)/1e6);

    colorMapping=0;  % 0 for gray, 1 to use RG for OFF/ON, 2 for time coherence

    frameDuration=.01; % seconds to collect spikes per displayed frame
    cs=1; % color scale: this number of spikes produces saturated color
    useColor=0;  % 0 for BW, 1 for RG, 2 for time=color
    usePolarity=1; % 1 to show spike polarity
    backwardsEnabled=0;
    rewindToStartEnabled=0;
    loopEnabled=1;
    pauseEnabled=0;
    markStart=abststart;
    markEnd=abstend;
    stopEnabled=0;
    renderer='OpenGL';
    jetMap=buildJetMap(cs);
    textEnabled=1;
    sliderMoved=0;
    recordMovieEnabled=0;
    buttonDown=0;
    rotation=0;

    % init figure
    if isempty(playbackFigure), playbackFigure=10; end;
    playbackFigure=figure;
    set (gcf, 'DoubleBuffer','on','Renderer',renderer);
    %set (gcf, 'MenuBar', 'none');
    set(gcf,'KeyPressFcn','playRecordedObject(''key'');');
    set(gcf,'WindowButtonDownFcn','playRecordedObject(''buttondown'');');
    set(gcf,'WindowButtonUpFcn','playRecordedObject(''buttonup'');');
    set(gcf,'DeleteFcn','playRecordedObject(''stop'');');
    timeSlider=uicontrol(playbackFigure,'Style','slider','position',[20,20,300,20]);
    set(timeSlider,'Callback','playRecordedObject(''sliderMoved'');');   
    %set(timeSlider,'ButtonDownFcn','playRecordedObject(''buttondown'');');
    im=image;
end
if nargin==1 & strcmp(cmd,'buttondown'),
    %disp buttondown
    buttonDown=1;
    return
elseif nargin==1 & strcmp(cmd,'buttonup'),
%     disp buttonup
    buttonDown=0;
    p=get(gca,'CurrentPoint');
    p=p(1,1:2);
    p=round(p);
    p(1)=max(1,p(1)); p(1)=min(d,p(1));
    p(2)=max(1,p(2)); p(2)=min(d,p(2));
    selectedPixel=p
    return
elseif nargin==1 & strcmp(cmd,'key'),
    k=get(gcf,'currentkey');
    switch k,
        case 'h'
            disp 'c cycle display color mapping'
            disp 'p toggle polarity enabled'
            disp 'uparrow/+/= increase contrast'
            disp 'downarrow/- decrease constrast'
            disp 'space pause/resume'
            disp 'r rewind to start'
            disp 'f faster (longer frames)'
            disp 's slower (shorter frames)'
            disp 'b toggle play backwards'
            %disp '1,2 mark start and end points'
            disp 'g cycle renderer'
            disp 'm toggle start/stop recording avi Movie'
            disp 'l toggle Loop through start and end points'
            disp 't Toggle text enabled (speeds up rendering)'
            disp 'o rOtate 90 deg. each press rotates another 90deg'
            disp 'x exit'
        case 'c'
            colorMapping=colorMapping+1;
            if colorMapping>2, colorMapping=0; end;
            if colorMapping==0,
                cs=grayColorCs; % restore
            end
            if colorMapping==2,
                grayColorCs=cs; % save for restoring it
                cs=63; % because cs=1 doesn't give anything sensible
                jetMap=buildJetMap(cs);
            end;
        case 'g'
            switch renderer,
                case 'painters',
                    renderer='OpenGL'
                case 'OpenGL',
                    renderer='zbuffer'
                case 'zbuffer',
                    renderer='painters'
            end
            set(gcf,'Renderer',renderer);
        case 'p'
            usePolarity=~usePolarity;
        case {'equal','add','uparrow'} % more contrast
            cs=cs-1; if cs<1, cs=1; end
            jetMap=buildJetMap(cs);
        case {'hyphen','subtract','downarrow'}
            cs=cs+1; if cs>64, cs=64; end
            jetMap=buildJetMap(cs);
        case 's' % slower (shorter frames)
            frameDuration=frameDuration/2;
            if frameDuration<1e-5, frameDuration=1e-5; end
        case 'f' % faster (longer frames)
            frameDuration=frameDuration*2;
            if frameDuration>10, frameDuration=10; end
        case 'b' % toggle direction
            backwardsEnabled=~backwardsEnabled;
        case 'space'
            pauseEnabled=~pauseEnabled;
        case 'r'
            rewindToStartEnabled=1;
            %         case '1'
            %             markStart=tstart;
        case 'm'
            recordMovieEnabled=~recordMovieEnabled;
            if recordMovieEnabled,
                mov=avifile('events.avi');
            else,
                mov=close(mov);
            end

            %         case '2'
            %             markEnd=tend;
        case 'l'
            loopEnabled=~loopEnabled;
        case 't'
            textEnabled=~textEnabled
        case 'x'
            stopEnabled=1;
            return
        case 'o'
            rotation=rotation+1; 
            if rotation>3, rotation=0; end
    end
    return
elseif nargin==1 & strcmp(cmd,'stop'),
    stopEnabled=1;
    return
elseif nargin==1 & strcmp(cmd,'sliderMoved'),
    sliderMoved=1;
    sliderValue=get(timeSlider,'value');
    return
end

% we make time windows, and for each spike in the window, we decode the x
% and y addresses and polarity and add it to an empty 64x64 matrix that
% gets plotted with a fixed brightness scaling.

tstart=abststart;
tend=tstart+frameDuration/tick;
startin=1;
endin=1;

while ~stopEnabled,
    %double(tend)<double(ae(2,end));
    if stopEnabled,
        break;
    end

    if sliderMoved,
        sliderMoved=0;
        backwardsEnabled=0;
        v=sliderValue; % get(timeSlider,'value');
%         fprintf('slider value=%f\n',v);
        tstart=abststart+(abstend-abststart)*v;
        tend=tstart+frameDuration/tick;
        in=find(ae(2,:)<=tstart);
        if isempty(in), in=numEvents; end
        startin=in(end);
        endin=startin;
        continue
    elseif ~buttonDown,
        v=(tstart-abststart)/(abstend-abststart);
        if v<0, v=0; elseif v>1, v=1; end
        set(timeSlider,'value',v);
    end

    if rewindToStartEnabled,
        backwardsEnabled=0;
        tstart=abststart;
        tend=tstart+frameDuration/tick;
        startin=1;
        endin=1;
        rewindToStartEnabled=0;
        continue
    end

    if pauseEnabled,
        v=(tstart-abststart)/(abstend-abststart);
        if v<0, v=0; elseif v>1, v=1; end
        if ~stopEnabled,  set(timeSlider,'value',v); end
        drawnow;
        pause(.1);
        continue;
    end

    if tstart>=abstend %| tstart>markEnd,
        title 'at end';
        drawnow
        pause(.3);
        if loopEnabled,
            tstart=abststart;
            tend=tstart+frameDuration/tick;
            startin=1;
            endin=1;
        else
            continue;
        end
    end
    if tstart<abststart %| tstart<markStart,
        title 'at start';
        drawnow
        pause(.3);
        if loopEnabled,
            backwardEnabled=0; % go forwards from start
            tstart=abststart;
            tend=tstart+frameDuration/tick;
            startin=1;
            endin=1;
        else
            continue
        end
    end

    %mask aer addresses to ON and OFF address-strings
    % find spikes in frame window, using pre-sorted timestamps


    if ~backwardsEnabled,
        while endin<numEvents && ae(2,endin)<tend,
            endin=endin+1;
        end
        addr=ae(1,startin:endin);
        ts=(ae(2,startin:endin));
        startin=endin+1;
    else
        while endin>1 && ae(2,endin)>tstart,
            endin=endin-1;
        end
        addr=ae(1,endin:startin);
        ts=(ae(2,endin:startin));
        startin=endin;
    end

    [x,y,pol]=extractObjectEventsFromAddr(addr);

    switch colorMapping,
        case 0,            % each addr sums into frame according to coloring choice
            fr=zeros(d,d,3);
            for i=1:length(x),
                fr(y(i)+1,x(i)+1,:)=fr(y(i)+1,x(i)+1,:)+pol(i);
                if ~isempty(selectedPixel),
                    if x(i)==selectedPixel(1) && y(i)==selectedPixel(2),
                        playSpikeSound;
                    end
                end
            end
            % events have summed +1 for on, -1 for off.
            % there are 2*cs+1 0-1 levels, black is 0, white is 1, and gray is .5. thus 0->.5 always
            fr=max(0,min(1,.5+fr/(2*cs)));
        case 1,
            fr=zeros(d,d,3);
            p=pol/2+.5+1;
            for i=1:length(x),
                fr(y(i)+1,x(i)+1,p(i))=fr(y(i)+1,x(i)+1,p(i))+1;
            end
            fr=max(0,min(1,fr/cs));
        case 2,
            fr=zeros(d,d,3); % start at black
            if length(ts)>0, % to make sure to correctly deal with no events
                t=double(ts); % timestamps
                mint=t(1); maxt=t(end);
                t=(t-mint)/(maxt-mint+eps); % t are relative times normalized during 'frame'
                t=fix(1+floor(cs*t));% index into colormap according to temporal offset in 'frame'
                for i=1:length(x),
                    fr(y(i)+1,x(i)+1,:)=jetMap(t(i),:);
                end
            end
    end
    if rotation>0,
        fr(:,:,1)=rot90(fr(:,:,1),rotation);
        fr(:,:,2)=rot90(fr(:,:,2),rotation);
        fr(:,:,3)=rot90(fr(:,:,3),rotation);
    end
    set(im,'CData',fr); %image(fr);
    %mesh(fr);
    set(gca,'zlim',[-cs,cs],'xlim',[1,d],'ylim',[1,d],'ydir','normal','xticklabel',[],'yticklabel',[]);
    axis square
    if textEnabled,
        if ~backwardsEnabled,
            dirString=' Fwd';
        else
            dirString=' Back';
        end
        tnow=(tstart-abststart)*tick;
        n=length(addr);
        s=sprintf('%.3fms@%.3fs, %d evts, %.0fkeps, FS=%d evts, %s',frameDuration*1000,tnow,n,n/frameDuration/1e3,cs,dirString);
        title(s);
    end
    drawnow; % force display

    if recordMovieEnabled,
        F=getframe(gca);
        mov=addframe(mov,F);
    end

    %pause
    % next frames times
    if ~backwardsEnabled,
        tstart=tstart+frameDuration/tick;
        tend=tstart+frameDuration/tick;
    else
        tstart=tstart-frameDuration/tick;
        tend=tstart+frameDuration/tick; % plays frames backward, but each frame is still forwards
    end
end % main loop
cleanup(playbackFigure);
return

function cleanup(figure)
delete(figure);
return

function jm=buildJetMap(cs)
jm=[0 0 0;jet(cs)];
return

function playSpikeSound
persistent spikeSoundSamples
if isempty(spikeSoundSamples),
    x=linspace(0,pi,100);
    y=sin(x).^4;
    z=gradient(y,x);
    z=z/max(z);
    spikeSoundSamples=z;
end
sound(spikeSoundSamples,8192,8);
return
