function showLiveRetina(arg,arg2)
%function showLiveRetina(arg)
% Hot keys are
% c cycle display color scheme'
% p toggle polarity
% up/down increase/decrease contrast
% l toggle logging
% space pause/resume
% g cycle graphics renderer between openGL, painters, and
% zbuffer
% a plAyback last logged events with playRecordedRetina
% u toggle sUbsample enabled
% t toggle text enabled (speeds rendering)
% u toggle subsampling (speeds rendering)
% e show median center of mass box
% x exit
%
% shows live retina output using usb interface, records events, etc
% just run without arguments to display retina output
% type h for help.
% by default uses openGL for rendering speed, g to toggle if there is a problem.
% a wireless network card can drastically reduce performance.
% author tobi feb 2005


persistent cs stopEnabled frameDuration colorMapping retinaSizeX retinaSizeY
persistent usePolarity reconstructionEnabled reconstructionTau
persistent loggingEnabled allAddr allTs
persistent pauseEnabled numEventsCaptured timeCaptured frameDelay renderer jetMap grayColorCs
% persistent fr im
persistent selectedPixel pixelIsSelected
persistent logFileName logTextFileEnabled logTextFile logTextFilename
persistent subsampleEnabled skipThreshold
persistent textEnabled
persistent displayRate

global retina

% put here variable defaults that are NOT changed by keystroke callbacks
% (if you put them here, they will be overwritten on each keystroke)

frameDelay=1/1000; % %1/200; % 0; % 1/50; %1/100; % delay between each rendering cycle, use for efficient timeslicing and to not use all CPU
skipThreshold=4000; % threshold for number of events to start skipping first 1, then 2, etc

%playSpikeSound(1);
if nargin==0 || nargin==2,
    % since callbacks from keystrokes are reentrant, this section only gets
    % called on initialization. put stuff here that takes time or that you
    % only want to do once. if the variables must be visible to the
    % callbacks (e.g. settings like textEnabled) then they must be
    % persistent variables. otherwise they will only be visible to main
    % rendering loop.
    % customizeable variables. these can be changed for another retina of
    % different size and address encoding
    if isempty(retina)
        retina='tmpdiff128';
    end

    if nargin==2 && strcmp(arg,'chip')
        retina=arg2;
    end

    if strcmp(retina,'tmpdiff64'),
        retinaSizeX=64; % size of retina (this is not general, just for plot limits)
        retinaSizeY=64; % size of retina (this is not general, just for plot limits)
        retinaEventExtractorFunction=@extractRetinaEventsFromAddr; %'extractRetinaEventsFromAddr';  % this is function that returns x,y,pol from AE, change for different retina, e.g. tmpdiff128
    elseif strcmp(retina,'tmpdiff128'),
        retinaSizeX=128; % size of retina (this is not general, just for plot limits)
        retinaSizeY=128; % size of retina (this is not general, just for plot limits)
        retinaEventExtractorFunction=@extractRetina128EventsFromAddr; %'extractRetinaEventsFromAddr';  % this is function that returns x,y,pol from AE, change for different retina, e.g. tmpdiff128
    elseif strcmp(retina,'doubleLine'),
        retinaSizeX=64; % size of retina (this is not general, just for plot limits)
        retinaSizeY=2; % size of retina (this is not general, just for plot limits)
        retinaEventExtractorFunction=@extractDoubleLineEventsFromAddr; %'extractRetinaEventsFromAddr';  % this is function that returns x,y,pol from AE, change for different retina, e.g. tmpdiff128
    else
        error([retina ' is not a defined retina chip']);
    end
    eventExtractor=[];
    try
        eventExtractor=getEventExtractor(retina);  % get the java class that extracts x,y,pol from ae (this speeds up things a lot)
    catch
        warning(['couldn''t get java event extractor for ' retina ', will be slightly slower to render']);
    end

    % determine connected interface
    try
        usbaemon();
        usbFunction=@usbaemon; %'usbaemon';
        fprintf('using usbaemon (SiLabs USBXPress)\n');
    catch
        try
            usb2aemon();
            usbFunction=@usb2aemon; %'usb2aemon';
            fprintf('using usb2aemon (Cypress FX2)\n');
        catch
            disp(lasterr);
            return
        end
    end
    cs=2; % color scale: this number of spikes produces saturated color
    colorMapping=0;  % 0 for gray, 1 to use RG for OFF/ON, 2 for time coherence
    usePolarity=1; % 1 to show spike polarity
    reconstructionEnabled=0; % set to 1 to reconstruct from events instead of displaying each 'frame' independently
    reconstructionTau=.2; % time constant of fading memory of past state of pixels
    zscale=cs; % scale for gray level
    stopEnabled=0; %used to stop loop
    loggingEnabled=0; % used to flag event logging
    subsampleEnabled=0;  % enables subsampling to hold rendering time down
    allAddr=[];
    allTs=[];
    numEventsCaptured=[]; timeCaptured=[];
    pauseEnabled=0;
    renderer='OpenGL';  % change this default if your machine doesn't have OpenGL support or it works poorly
    jetMap=buildJetMap(cs);
    fr=zeros(retinaSizeY,retinaSizeX,3);
    textEnabled=1;  % set this 0 by default if you don't want stats, really speeds up rendering
    pixelIsSelected=0;
    logTextFileEnabled=0;  % set to 1 to continuously log data to a text file (instead of to matlab memory and later to disk)
    displayRate=ones(1,30); % holds rendering times for framerate computation
end

if nargin==1 && strcmp(arg,'stop'),
    stopEnabled=1;
end

if nargin==1 && strcmp(arg,'buttonup'),
    p=get(gca,'CurrentPoint');
    p=p(1,1:2); % get first row, first two elements; these are x,y position
    if p(1)<-1 || p(1)>retinaSizeY+1 || p(2)<-1 || p(2)>retinaSizeY+1,
        selectedPixel=[];
        fprintf('cleared pixel selection\n');
        pixelIsSelected=0;
    else
        p=round(p);
        p(1)=max(1,p(1)); p(1)=min(retinaSizeX,p(1));
        p(2)=max(1,p(2)); p(2)=min(retinaSizeY,p(2));
        selectedPixel=p-1;
        fprintf('selected pixel %d,%d\n',selectedPixel(1),selectedPixel(2));
        pixelIsSelected=1;
    end
    return
elseif nargin==1 && strcmp(arg,'key'),
    k=get(gcf,'currentkey');
    switch k,
        case 'h'
            helpwin showLiveRetina
        case 'g'
            switch renderer,
                case 'painters',
                    renderer='OpenGL'
                case 'OpenGL',
                    renderer='zbuffer'
                case 'zbuffer',
                    renderer='painters'
            end
            set(gcf,'Renderer',renderer);
        case 'c'
            colorMapping=colorMapping+1;
            if colorMapping>3, colorMapping=0; end;
            if colorMapping==0,
                cs=grayColorCs; % restore
            end
            if colorMapping==2,
                grayColorCs=cs; % save for restoring it
                cs=63; % because cs=1 doesn't give anything sensible
                jetMap=buildJetMap(cs);
            end;
        case 'p'
            usePolarity=~usePolarity;
        case {'equal','add','uparrow','rightarrow'}
            cs=cs-1; if cs<1, cs=1; end
            jetMap=buildJetMap(cs);
        case {'hyphen','subtract','downarrow','leftarrow'}
            cs=cs+1; if cs>64, cs=64; end
            jetMap=buildJetMap(cs);
        case 'space'
            pauseEnabled=~pauseEnabled;
        case 'l'
            loggingEnabled=~loggingEnabled;
            if ~loggingEnabled, % logging ended
                logFilename=['events',datestr(now,30)];
                %                save(logFilename, 'allAddr', 'allTs');
                clear eventExtractor ;  % can't serialize java object
                save(logFilename);
                eventExtractor=getEventExtractor(retina);
                fprintf('events saved to mat file %s.mat\n',logFilename);
                if logTextFileEnabled,
                    fprintf('events saved to text file %s\n',logTextFilename);
                    fprintf(logTextFile,'#logging ended %s\n',datestr(now));
                    fclose(logTextFile);
                end
            else % logging started
                fprintf('started logging events\n');
                if logTextFileEnabled,
                    logTextFilename=['events',datestr(now,30),'.txt'];
                    logTextFile=fopen(logTextFilename,'w');
                    fprintf(logTextFile,'#events logged to file %s from showLiveRetina.m\n',logTextFilename);
                    fprintf(logTextFile,'#logging started %s\n',datestr(now));
                    fprintf(logTextFile,'#addr(16 bit) timeStamp(32 bit us)\n',logFileName);
                end
                allAddr=[]; allTs=[];
            end
        case 'a'
            % playback of last recorded data
            if ~isempty(logFileName),
                playRecordedRetina('playFile',logFileName);
            else
                playRecordedRetina;
            end
        case 'x'
            stopEnabled=1;
            title('stopping...','color','r');
            drawnow;
            return
        case 'u'
            % toggle subsampling to speed up frame rate for large spike
            % counts
            subsampleEnabled=~subsampleEnabled;
        case 't'
            textEnabled=~textEnabled;
            if ~textEnabled, title(''); end
    end
    return
end

figure(1);
set(gcf,'name','Retina');
clf;
set (gcf, 'DoubleBuffer','on','Renderer',renderer);
set (gcf, 'MenuBar', 'none');
set(gcf,'keypressfcn','showLiveRetina(''key'')')
set(gcf,'WindowButtonUpFcn','showLiveRetina(''buttonup'');');
bg=.2;
set(gcf,'color',[bg,bg,bg]);
set(gcf,'closerequestfcn','showLiveRetina(''stop'');');
set(gcf,'backingstore','off');
im=image(fr);
connectionError=0; % used to track error status in order to clear error title on reconnect
set(gca,'zlim',[-zscale,zscale],'xlim',[.5,retinaSizeX+.5],'ylim',[.5,retinaSizeY+.5],'ydir','normal','xticklabel',[],'yticklabel',[]);
tic;  % start frame timer
startNow=now;
while ~stopEnabled,
    if stopEnabled,
        break;
    end
    if pauseEnabled,
        usbFunction();  % to make sure to maintain USB connection even if retina is off
        pause(.3);
        continue;
    end;
    try
        % capture events from usb interface; this call is eval to allow for multiple interfaces
        [addr,ts,overrun]=usbFunction(); %eval(['[addr,ts,overrun]=' usbFunction ';']);
        if connectionError,
            connectionError=0;
            title('Connected OK','color','g');
            drawnow;
        end
    catch
        title(lasterr,'color','r');
        drawnow;
        connectionError=1;
        pause(1);
        continue;
    end
    if loggingEnabled,
        numEventsCaptured=[numEventsCaptured length(addr)];
        timeCaptured=[timeCaptured now-startNow];
        allAddr=[allAddr, addr];
        allTs=[allTs,ts];
        if logTextFileEnabled,
            fprintf(logTextFile,'%d %d\n',[double(addr);double(ts)]);
        end
    end
    %extract x,y,pol of events using this chip's extractor function
    if length(addr)>0,
        numEvents=length(addr);
        skipBy=1;
        if subsampleEnabled,
            if numEvents>skipThreshold,
                skipBy=ceil(numEvents/skipThreshold); % skip every 1,2,3...
                subsamplingOccurred=1;
            else
                subsamplingOccurred=0;
            end
        end
        sampleIndex=1:skipBy:numEvents;

        if ~isempty(eventExtractor),  % it there is a java class for extracting event meaning, use it
            eventExtractor.extract(addr(sampleIndex));
            x=eventExtractor.x;
            y=eventExtractor.y;
            pol=eventExtractor.pol;
        else
            [x,y,pol]=retinaEventExtractorFunction(addr(sampleIndex)); % only extract the sampled events
        end

        m=length(sampleIndex);
        try
            switch colorMapping,
                case 0,      % on/off white/black mapping      % each addr sums into frame according to coloring choice
                    fr=fr*0;
                    for i=1:m,
                        fr(y(i)+1,x(i)+1,1)=fr(y(i)+1,x(i)+1,1)+pol(i);
                        if pixelIsSelected && x(i)==selectedPixel(1) && y(i)==selectedPixel(2), playSpikeSound; end
                    end
                    % events have summed +1 for on, -1 for off.
                    % there are 2*cs+1 0-1 levels, black is 0, white is 1, and gray is .5. thus 0->.5 always
                    fr=max(0,min(1,.5+fr/(2*cs)));
                    fr(:,:,2)=fr(:,:,1);
                    fr(:,:,3)=fr(:,:,1); %gray
                    set(im,'CData',fr);
                case 1, % red/green off/on mapping
                    fr=fr*0;
                    p=pol/2+.5+1;
                    for i=1:m,
                        fr(y(i)+1,x(i)+1,p(i))=fr(y(i)+1,x(i)+1,p(i))+1;
                        if pixelIsSelected && x(i)==selectedPixel(1) && y(i)==selectedPixel(2), playSpikeSound; end
                    end
                    fr=max(0,min(1,fr/cs));
                    set(im,'CData',fr);
                case 2, % color time mapping
                    fr=fr*0;
                    %fr=zeros(retinaSizeX,retinaSizeX,3); % start at black
                    t=double(ts); % timestamps
                    mint=t(1); maxt=t(end);
                    t=(t-mint)/(maxt-mint+eps); % t are relative times normalized during 'frame'
                    t=fix(1+floor(cs*t));% index into colormap according to temporal offset in 'frame'
                    for i=1:m,
                        fr(y(i)+1,x(i)+1,:)=jetMap(t(i),:);
                        if pixelIsSelected && x(i)==selectedPixel(1) && y(i)==selectedPixel(2), playSpikeSound; end
                    end
                    set(im,'CData',fr);
                case 3,  % grayscale time mapping
                    fr=fr*0;
                    %fr=zeros(retinaSizeX,retinaSizeX,3); % start at black
                    t=double(ts); % timestamps
                    mint=t(1); maxt=t(end);
                    t=(t-mint)/(maxt-mint+eps); % t are relative times normalized during 'frame' 0 is oldest, 1 is youngest
                    for i=1:m,
                        fr(y(i)+1,x(i)+1,:)=t(i);  % set grayscale
                        if pixelIsSelected && x(i)==selectedPixel(1) && y(i)==selectedPixel(2), playSpikeSound; end
                    end
                    set(im,'CData',fr);
            end
        catch % in case there is problem with sound card
            disp(lasterr);
            selectedPixel=[];
        end
    end
    % make title string here
    if textEnabled && length(addr)>0,
        dt=(double(ts(end))-double(ts(1)))/1e3; % dt in ms
        %         figure(22);
        %         plot(1:numEvents, ts);
        %         figure(1);
        if numEvents<2,
            rate=0;
        else
            rate=numEvents/(dt+eps);
        end
        if subsampleEnabled && subsamplingOccurred,
            subString='subsampled';
        else
            subString='';
        end
        if overrun,
            overrunText='overrun';
        else
            overrunText='';
        end
        frameTime=toc;
        displayRate=1/toc;
        tic;
        s=sprintf('dt=%.0f ms, %d evts %s %s (%.1f keps), %.0fHz, FS=%d evts',dt,numEvents,subString,overrunText,rate,displayRate,cs);
        if loggingEnabled,
            s=[s, ' logging'];
        end
        if overrun
            title(s,'color','r');
        else
            title(s,'color','g');
        end
        % debug ts
        %         t=double(ts);
        %         dt=diff(t);
        %         jt=find(dt>2^16);
        %         for i=1:length(jt),
        %             if i>1 & i<length(jt),
        %                 fprintf('wrap error at event %d, timestamps %X,%X,%X\n',jt(i),t(jt(i-1)),t(jt(i)),t(jt(i+1)));
        %             end
        %         end
        % end debug
        %     else
        %         s=sprintf('No events');
    end % title string
    axis square
    drawnow; % force display
    pause(frameDelay);


end % render loop
cleanup(usbFunction)
return

function cleanup(usbfunc)
info=functions(usbfunc);
name=info.function;
if strcmp(name,'usbaemon')
    clear usbaemon
elseif strcmp(name,'usb2aemon')
    usb2aemon('close');
end
delete(figure(1));
return

function jm=buildJetMap(cs)
jm=[0 0 0;jet(cs)];
return




