/* This console app tests the usbaer board by capturing events and printing them out.
	note that this file *must* be called something.cpp (and not something.c) to enable linking to the SiUSBXp.dll functions implicitly!!!
	-tobi Feb 2005
	
*/
// usbaemontest.cpp : Defines the entry point for the console application.
#include <windows.h>
#include "stdafx.h"
#include "SiUSBXp.h"	// customized version of SILabs header modified for C from C++

// also static so persistent
static DWORD deviceNum;	//	usb device number (will almost always be 0)
static HANDLE deviceHandle=NULL;	// handle returned to device by SI_Open. used by SI_Close and mexAtExit/CloseStream
static HMODULE lib=NULL;	// DLL SiUSBXp.dll


static void onExitClearError(void); // exit handler
void printErrorCodes(void);

void mexErrMsgTxt(const char *s){
	printf("%s\n",s);
	exit(1);
}

void onError(SI_STATUS status){
	printf("error status=0x%x\n",status);
	printErrorCodes();
}

void printErrorCodes(void){
	// Return codes
	printf("#define		SI_SUCCESS				0x00\n");
	printf("#define		SI_DEVICE_NOT_FOUND		0xFF\n");
	printf("#define		SI_INVALID_HANDLE			0x01\n");
	printf("#define		SI_READ_ERROR				0x02\n");
	printf("#define		SI_RX_QUEUE_NOT_READY		0x03\n");
	printf("#define		SI_WRITE_ERROR			0x04\n");
	printf("#define		SI_RESET_ERROR			0x05\n");
	printf("#define		SI_INVALID_PARAMETER		0x06\n");
	printf("#define		SI_INVALID_REQUEST_LENGTH	0x07\n");
	printf("#define		SI_DEVICE_IO_FAILED		0x08\n");
	printf("#define		SI_INVALID_BAUDRATE		0x09\n");
}


// static buffer for collected events
static unsigned char aeBuffer[SI_MAX_READ_SIZE+1];

// main function
// has exactly one function: to return available events
// if DLL is not loaded it will be
// if device is not opened, it will be
int main(int argc, char* argv[])
{
 	unsigned int j;
 	int numEvents;
 	unsigned short addr,ts;
 	int readyToRead;	// for rcv queue
	
     DWORD	numDevices;
     SI_STATUS status;
     char deviceString[SI_MAX_DEVICE_STRLEN];    
 	DWORD bytesRead, bytesInQueue,queueStatus;
	
	if(deviceHandle==NULL){ // we only open device once
		//printf("finding number of attached devices\n");
		status=(SI_GetNumDevices)(&numDevices);
		if(status!=SI_SUCCESS){
			printf("error SI_GetNumDevices, SI_STATUS=%d, numDevices=%d\n",status,numDevices);
			if(status==0xff){printf("SI_DEVICE_NOT_FOUND (USBXPress Device driver not installed, cable not connected, device not powered?)\n");}
			onError(status);
			mexErrMsgTxt("error SI_GetNumDevices");
		}	
		if(numDevices==0){
			mexErrMsgTxt("error SI_GetNumDevices no devices");
		}
		printf("%d device(s) found, choosing first one\n",numDevices);
		deviceNum=0;	// always choose first device
		

		deviceString[0]=0;
		status=(SI_GetProductString)(deviceNum,deviceString,SI_RETURN_DESCRIPTION);
		if(status!=SI_SUCCESS){
			onError(status);
			mexErrMsgTxt("error SI_GetProductString");
		}
		printf("product string: %s\n",deviceString); // this never returns something sensible
			
		status = (SI_SetTimeouts)(100,100); // set readtimeout=100ms, writetimeout to 100ms for recovery from hang
		if(status!=SI_SUCCESS){
			onError(status);
			mexErrMsgTxt("error SI_SetTimeouts");
		}
		//printf("setTimeouts OK\n");

		status = (SI_Open)(0, &deviceHandle);
		if(status!=SI_SUCCESS){
			onError(status);
			mexErrMsgTxt("error SI_Open");
		}
		printf("device opened, device handle=%d\n",deviceHandle);

		status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers on open
		if(status!=SI_SUCCESS){
			printf("error SI_FlushBuffers, SI_STATUS=%d\n",status,numDevices);
			onError(status);
			mexErrMsgTxt("error SI_FlushBuffers");
		}
		//printf("setTimeouts OK\n");
	}

    
    
    // write a message to stimulate tranmission of events
    //SI_STATUS SI_Write (HANDLE Handle, LPVOID Buffer, DWORD NumBytesToWrite,DWORD*NumBytesWritten);
/*    status=(SI_Write)(deviceHandle,&msg,1,&bytesWritten);
    if(status!=SI_SUCCESS){
    	printf("error writing message, status=%d\n",status);
		mexErrMsgTxt("error SI_Write");
	}
	printf("wrote msg to start xmit from device\n");	
*/
    	
	// check rcv queue
	// RX Queue status flags #define SI_RX_NO_OVERRUN 0x00 #define SI_RX_OVERRUN 0x01 #define SI_RX_READY 0x02
	readyToRead=0;
	bytesInQueue=0;
	while(!readyToRead){
		status=(SI_CheckRXQueue)(deviceHandle,&bytesInQueue,&queueStatus);
		if(status!=SI_SUCCESS){
	 		printf("error SI_CheckRXQueue: status=%d, queueStatus=%d\n",status,queueStatus);
			onError(status);
			mexErrMsgTxt("error SI_CheckRXQueue");
		}
		if(queueStatus&SI_RX_OVERRUN){
	 		printf("warning: SI_RX_OVERRUN, %d bytes in queue\n",bytesInQueue);
	 		readyToRead=1;
 		}
 		if(queueStatus&SI_RX_READY){
 			readyToRead=1;
 		}
		if(readyToRead==0)printf("not ready to rx yet\n");
	}
	printf("%d bytesInQueue\n",(int)bytesInQueue);
	if(bytesInQueue>65536){
		printf("%d bytes in queue, that's too many, flushing buffer\n", (int)bytesInQueue);
		status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers on open
		if(status!=SI_SUCCESS){
			printf("error SI_FlushBuffers, SI_STATUS=%d\n",(int)status,numDevices);
			onError(status);
			mexErrMsgTxt("error SI_FlushBuffers");
		}
		mexErrMsgTxt("error too many bytes in queue");
	}
	
	// read events
	//SI_STATUS SI_Read (HANDLE Handle, LPVOID Buffer, DWORD NumBytesToRead,DWORD*NumBytesReturned)
	bytesRead=0;
	printf("%d bytesInQueue\n",(int)bytesInQueue);
	if(bytesInQueue>0 && bytesInQueue<=65536){
		status=(SI_Read)(deviceHandle,aeBuffer,bytesInQueue,&bytesRead);
		if(status!=SI_SUCCESS){
			printf("error reading bytes: status=%d, bytesInQueue=%d\n",(int)status,(int)bytesInQueue);
			onError(status);
			mexErrMsgTxt("error SI_Read");
		}
		printf("read %d bytes\n",bytesRead);
		for(j=0;j<bytesRead;j++){
			printf("%d,",(int)aeBuffer[j]);
		}
		printf("\n");
	}
	printf("%d bytes read\n",(int)bytesRead);	
	numEvents=bytesRead/4;
//	printf("%d bytesRead, %d events\n",(int)bytesRead,(int)numEvents);
//	if(numEvents>16384){
//		printf("numEvents=%d, received more than 16k events, flushing buffers\n", numEvents);
//		status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers on open
//		if(status!=SI_SUCCESS){
//			printf("error SI_FlushBuffers, SI_STATUS=%d\n",status,numDevices);
//			onError(status);
//			mexErrMsgTxt("error SI_FlushBuffers");
//		}
//		mexErrMsgTxt("error too many events");
//	}
		
//	if(numEvents>0){
		/* Create a 2-by-n array of unsigned 16-bit integers. */
	
		/* Populate the real part of the created array. */
		// ae are lsb,msb, and so are counter values, i.e., least significant part is earlier in array
		// matrix is 2 high by numEvents wide, linear form addr0,ts0,addr1,ts1, etc.
		// elementSize=mxGetElementSize(plhs[0]);
//		printf("populating matrix\n");
		numEvents=0;
		printf("addr\ttimestamp\n");
		for(j=0;j<bytesRead;j+=4){
			// events come over big-endian, MSB comes first
			addr=aeBuffer[j+1]+((aeBuffer[j])<<8);
			ts=aeBuffer[j+3]+((aeBuffer[j+2])<<8);
			printf("%d\t%d\n",(int)addr,(int)ts);
			numEvents++;
		}

		onExitClearError();
		return 0;
}	// end of main


/* Here is the error function, which gets run when the MEX-file is
   cleared and when the user exits MATLAB. The mexAtExit function
   should always be declared as static. */
static void onExitClearError(void)
{
  SI_STATUS status;
  printf("calling SI_Close\n");
  if(SI_Close!=NULL && deviceHandle!=NULL) {
  		status = (SI_Close)(deviceHandle);
		if(status!=SI_SUCCESS){
			printf("error SI_Close: status=%d\n",status);
			onError(status);
			deviceHandle=NULL;
			mexErrMsgTxt("error SI_Close");
		}
	}
	printf("SI_Close successful\n");
	deviceHandle=NULL;
	
}

