// USBAEMonitor: implementations of the native methods in class USBAEMonitor.
/* 
source for USB AE Monitor java native method using simple Sevilla USB AE board and SiLabs USBXPress

 -tobi feb 2005
 */

#include "stdafx.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}


#define DEVICE_STILL_THERE_POLLING_INTERVAL_MS 1000
#define TIMEOUT_MS		0	// defines timeout for read, write. read will return with available events in this time
							// even if all are not available
#define TICK_US	1	// this is timer tick on USB AER microcontroller. See firmware in folder "Device"

// MS VC should compile this ".c" file with the plain C compiler based on ".c" extension, so no extern "C" is needed

// these are declared static to be invisible outside
static void closeDevice(void); // exit handler
static void onError(SI_STATUS);
static SI_STATUS flushBuffers(void);

#ifdef __cplusplus
extern "C" {
#endif

static jclass clazz=NULL;
static jfieldID addressesFieldID=NULL, timestampsFieldID=NULL, overrunOccuredFieldID=NULL;
// deviceHandle is static local var so that DLL exit handler could use it to close device	
static HANDLE deviceHandle;	// handle returned to device by SI_Open. used by SI_Close
static unsigned char aeBuffer[SI_MAX_READ_SIZE]; // static buffer for collected events
static unsigned short addressBuffer[SI_MAX_READ_SIZE/4]; // static buffer for collected events
static unsigned int timestampBuffer[SI_MAX_READ_SIZE/4]; // static buffer for collected events
static int numEvents;
static jboolean overrunOccured=JNI_FALSE;
static DWORD deviceNum;	//	usb device number (will almost always be 0)
static DWORD	numDevices;
static SI_STATUS status;
static char deviceString[SI_MAX_DEVICE_STRLEN];    
static unsigned int wrapAdd=0;		// static offset to add to spike timestamps
static unsigned short lastshortts=0;		// holds last 16 bit timestamp of previous event, to detect wrapping
static DWORD lastTimeCheckedDeviceThere;

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    initIDs
 * Signature: ()V
 */
JNIEXPORT jint JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_initIDs
(JNIEnv *env, jclass clazz){
	 // get field IDs in class for later use and cache the field IDs in static vars
/* at present there are no fields in object to reference
	if(addressesFieldID==NULL){
		addressesFieldID=(*env)->GetFieldID(env,clazz,"addresses","[S");
		if(addressesFieldID==NULL) return -1;
		timestampsFieldID=(*env)->GetFieldID(env,clazz,"timestamps","[I");
		if(timestampsFieldID==NULL) return -1;
		overrunOccuredFieldID=(*env)->GetFieldID(env,clazz,"overrunOccured","Z");
		if(overrunOccuredFieldID==NULL) return -1;
	}
*/
	return 0; //success
}

/*

	We have already captured numEvents events into buffers addressBuffer and timestampBuffer.
	Here we construct the new java array, get a pointer to a native version of it, copy
	the events to the native version and release the array, returning it to the caller.

 */

 /*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    getAddresses
 * Signature: ()[S
 */
JNIEXPORT jshortArray JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeGetAddresses
(JNIEnv *env, jobject obj){
 	unsigned short*aePtr;
	jshortArray jaddresses=NULL;
	int i;

	if(numEvents==0) return NULL;

	// allocate java arrays to hold the events
	jaddresses=(*env)->NewShortArray(env,numEvents);
	if(jaddresses==NULL) return NULL; // memory?


	// get native pointers hopefully directly (or perhaps copy) to these memory segments
	// do not make any JNI calls between Get and Release of these arrays
	aePtr=(*env)->GetPrimitiveArrayCritical(env,jaddresses,0);
	if(aePtr==0) return NULL; // out of memory

	// copy events to the native mirrors of arrays
	for(i=0;i<numEvents;i++){
		aePtr[i]=(short)addressBuffer[i];
	}

	// release hold on arrays
	(*env)->ReleasePrimitiveArrayCritical(env,jaddresses,aePtr,0);
	return jaddresses;
}

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    getTimestamps
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeGetTimestamps
(JNIEnv *env, jobject obj){

	unsigned int *tsPtr;	// pointer to 32 bit returned timetamp arrar
	jintArray jtimestamps=NULL;
	int i;

	if(numEvents==0) return NULL;

	jtimestamps=(*env)->NewIntArray(env,numEvents);
	if(jtimestamps==NULL) return NULL; // memory?

	// get native pointers hopefully directly (or perhaps copy) to these memory segments
	// do not make any JNI calls between Get and Release of these arrays
	tsPtr=(*env)->GetPrimitiveArrayCritical(env,jtimestamps,0);
	if(tsPtr==0) return NULL; // out of memory

	// copy events to the native mirrors of arrays
	for(i=0;i<numEvents;i++){
		tsPtr[i]=(int)timestampBuffer[i];
	}

	// release hold on arrays
	(*env)->ReleasePrimitiveArrayCritical(env,jtimestamps,tsPtr,0);

	return jtimestamps;
}

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    overrunOccured
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeOverrunOccured
(JNIEnv *env, jobject obj){
	return (jboolean)overrunOccured;
}

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    open
 * Signature: ()V
 */
JNIEXPORT jint JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeOpen
(JNIEnv *env, jobject obj){
	// start actual use of functions from library, finally!!!
	if(deviceHandle==NULL){ // we only open device once
		//printf("USBAEMonitor.c: finding number of attached devices\n");
		status=(SI_GetNumDevices)(&numDevices);
		if(status!=SI_SUCCESS){
			printf("USBAEMonitor.c: error SI_GetNumDevices, numDevices=%d, SI_STATUS=0x%x\n",numDevices,status);
			if(status==0xff){printf("USBAEMonitor.c: SI_DEVICE_NOT_FOUND \n(USBXPress Device driver not installed, cable not connected, device not powered?)\n");}
			onError(status);
			printf("USBAEMonitor.c: error SI_GetNumDevices\n");
			fflush(stdout);
			return -1;
		}	
		if(numDevices==0){
			printf("USBAEMonitor.c: error SI_GetNumDevices no devices\n");
			fflush(stdout);
			return -1;
		}
//		printf("USBAEMonitor.c: %d device(s) found, choosing first one\n",numDevices);
		deviceNum=0;	// always choose first device
		

		deviceString[0]=0;
		status=(SI_GetProductString)(deviceNum,deviceString,SI_RETURN_DESCRIPTION);
		if(status!=SI_SUCCESS){
			onError(status);
			printf("USBAEMonitor.c: error SI_GetProductString\n");
			fflush(stdout);
			return -1;
		}
		printf("USBAEMonitor.c: USB AE Mon product string: %s\n",deviceString); // this never returns something sensible
		fflush(stdout);
		
		status = (SI_SetTimeouts)(TIMEOUT_MS,TIMEOUT_MS); // set readtimeout=100ms, writetimeout to 100ms for recovery from hang
															// write timeout set even though we never write (so far)
		if(status!=SI_SUCCESS){
			onError(status);
			printf("USBAEMonitor.c: error SI_SetTimeouts");
			fflush(stdout);
			return -1;
		}
		//printf("USBAEMonitor.c: setTimeouts OK\n");

		status = (SI_Open)(0, &deviceHandle);
		if(status!=SI_SUCCESS){
			onError(status);
			printf("USBAEMonitor.c: error SI_Open");
			fflush(stdout);
			return -1;
		}
//		printf("USBAEMonitor.c: device opened, device handle=%d\n",deviceHandle);
		wrapAdd=0; // reset timestamps
		lastTimeCheckedDeviceThere=GetTickCount();
		return flushBuffers();
    
	}else{
		return -1; // device opened already, have a device handle
	}
}

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    close
 * Signature: ()V
 */
JNIEXPORT jint JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeClose
(JNIEnv *env, jobject obj){
	closeDevice();
	return 0;
}

/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    getNumEventsAcquired
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeGetNumEventsAcquired
(JNIEnv *env, jobject obj){
	return (jint)numEvents;
}



/*
 * Class:     ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor
 * Method:    acquireAvailableEventsFromDriver
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_ch_unizh_ini_caviar_usbaemonitor_USBAEMonitor_nativeAcquireAvailableEventsFromDriver
(JNIEnv *env, jobject obj){
	unsigned int ui;
 	int eventCounter;
 	unsigned short addr,*aePtr;
	unsigned short shortts;	// raw 16 bit timestamp from device
	unsigned int *tsPtr;	// pointer to 32 bit returned timetamp arrar
	//int readyToRead;	// for rcv queue
	//int bytes_to_copy;

	

	DWORD bytesRead; // unsigned long
	DWORD bytesInQueue;
	DWORD queueStatus; // ,bytesWritten;

	// DLL is left resident and device is left open between native method calls

    
	// every so often, check if device is still actually there
	// if there, just continue.
	// if error or no device found, return error and clear device handle. 
	// user code can try/catch around calls to maintain flexible connection
	if(GetTickCount()-lastTimeCheckedDeviceThere>DEVICE_STILL_THERE_POLLING_INTERVAL_MS){
		lastTimeCheckedDeviceThere=GetTickCount();
		status=(SI_GetNumDevices)(&numDevices);
		if(status!=SI_SUCCESS){
			printf("error SI_GetNumDevices, numDevices=%d, SI_STATUS=0x%x\n",numDevices,status);
			if(status==0xff){printf("SI_DEVICE_NOT_FOUND \n(USBXPress Device driver not installed, cable not connected, device not powered?)\n");}
			onError(status);
			deviceHandle=NULL;
			fflush(stdout);
			return -1;  // returns error
		}	
		if(numDevices==0){
			deviceHandle=NULL;
			printf("error SI_GetNumDevices no devices"); // returns error
			fflush(stdout);
			return -1;
		}
	}

	// we read all bytes available *if* driver says its ok to read anything
	// if driver isn't ready to read, we just return 0 events
	// if there is an overrun just detect it and return the events that have been stored

	status=(SI_CheckRXQueue)(deviceHandle,&bytesInQueue,&queueStatus);
	if(status!=SI_SUCCESS){
	 	printf("USBAEMonitor.c: error SI_CheckRXQueue: status=0x%x queueStatus=0x%x\n",status,queueStatus);
		onError(status);
		printf("USBAEMonitor.c: error SI_CheckRXQueue");
		fflush(stdout);
		return -1;
	}

	overrunOccured=queueStatus&SI_RX_OVERRUN?JNI_TRUE:JNI_FALSE;
	
	// read events
	//SI_STATUS SI_Read (HANDLE Handle, LPVOID Buffer, DWORD NumBytesToRead,DWORD*NumBytesReturned)
	bytesRead=0;

	// if there is an overrun, then bytesInQueue is set to 0. we set it to max anyhow to get all available events

	bytesInQueue=overrunOccured?SI_MAX_READ_SIZE:bytesInQueue;
	status=(SI_Read)(deviceHandle,aeBuffer,bytesInQueue,&bytesRead);
	if(status!=SI_SUCCESS){
		if(status&SI_RX_QUEUE_NOT_READY) {
			numEvents=0;
			return 0;
		}else{
			printf("USBAEMonitor.c: error reading bytes: status=%d, bytesInQueue=%d\n",(int)status,(int)bytesInQueue);
			onError(status);
			printf("USBAEMonitor.c: error SI_Read\n");
			numEvents=0;
			fflush(stdout);
			return -1;
		}
	}

	if(overrunOccured) flushBuffers(); // clears the flag

//		printf("USBAEMonitor.c: read %d bytes\n",bytesRead);
//		for(ui=0;ui<bytesRead;ui++){
//			printf("USBAEMonitor.c: %d,",(int)aeBuffer[ui]);
//		}
//		printf("USBAEMonitor.c: \n");
//	printf("USBAEMonitor.c: %d bytes read\n",(int)bytesRead);

	if (bytesRead%4!=0){
		printf("USBAEMonitor.c: read %d bytes, which is not divisible by 4, something's wrong, closing\n", bytesRead);
		closeDevice();
		printf("USBAEMonitor.c: error in reading events");
		numEvents=0;
		fflush(stdout);
		return -1;
	}
	numEvents=bytesRead/4;
//	printf("USBAEMonitor.c: %d bytesRead, %d events\n",(int)bytesRead,(int)numEvents);
	
//		printf("USBAEMonitor.c: creating output matrix 2 x %d\n",numEvents);

	/* 
	First, grab bytes from driver, unpack addresses and timestamps (unwrapping these) into local static buffers.

	ae and timestamps are sent from USB device in BIG-ENDIAN format. MSB comes first,
	The addresses are simply copied over, and the timestamps are unwrapped to make uint32 timestamps.
	wrapping is detected when the present timestamp is less than the previous one.
	then we assume the counter has wrapped--but only once--and add into this and subsequent
	timestamps the wrap value of 2^16. this offset is maintained and increases every time there 
	is a wrap. hence it integrates the wrap offset.
	the timestamp is returned in 1us ticks, although the microcontroller uses 2us ticks.
	this conversion is to make values more compatible with other CAVIAR software components.
	
	from http://www.codeproject.com/cpp/endianness.asp
	Little-endian: Least significant byte is stored at the lowest byte address.
	The Intel x86 family and Digital Equipment Corporation architectures (PDP-11, VAX, Alpha) 
	are representatives of Little-Endian.

	8051 Keil compiler, however, is big-endian.
	*/
	eventCounter=0;
	aePtr=addressBuffer;
	tsPtr=timestampBuffer;
	for(ui=0;ui<bytesRead;ui+=4){
		addr=aeBuffer[ui+1]+((aeBuffer[ui])<<8); // remember addr is sent big-endian
		aePtr[eventCounter]=addr;	// simply copy address
		{	// if we're returning timestamps
			shortts=aeBuffer[ui+3]+((aeBuffer[ui+2])<<8); // this is 16 bit value of timestamp in 2us tick
			if(shortts<lastshortts){
				wrapAdd+=0x10000;	// if we wrapped then increment wrap value by 2^16
//					printf("USBAEMonitor.c: event %d wraps: lastshortts=0x%X, shortts=0x%X, wrapAdd now=0x%X, final ts/2=0x%X, final ts=0x%X\n",
//						eventCounter+1,lastshortts,shortts,wrapAdd,(shortts+wrapAdd),(shortts+wrapAdd)*2);
			}
			tsPtr[eventCounter]=(shortts+wrapAdd)*TICK_US; //add in the wrap offset and convert to 1us tick
			lastshortts=shortts;	// save last timestamp
		}
		eventCounter++;
	}
//	printf("USBAEMonitor.c: %d events\n",eventCounter);

	return numEvents; // successful
		
}	// end of mexFunction

/* Here is the error function, which gets run when the MEX-file is
   cleared and when the user exits MATLAB. The mexAtExit function
   should always be declared as static. */
static void closeDevice(void)
{
  SI_STATUS status;
  //printf("USBAEMonitor.c: calling SI_Close\n");
  if(deviceHandle!=NULL) {
  		status = (SI_Close)(deviceHandle);
		if(status!=SI_SUCCESS){
			printf("USBAEMonitor.c: error SI_Close\n");
			onError(status);
			deviceHandle=NULL;
			fflush(stdout);
		}
		printf("USBAEMonitor.c: SI_Close successful\n");
	}
	deviceHandle=NULL;
}

void onError(SI_STATUS status){
	printf("USBAEMonitor.c: error status=0x%x\n",status);
	printf("SI_SUCCESS				0x00\n");
	printf("SI_DEVICE_NOT_FOUND		0xFF\n");
	printf("SI_INVALID_HANDLE			0x01\n");
	printf("SI_READ_ERROR				0x02\n");
	printf("SI_RX_QUEUE_NOT_READY		0x03\n");
	printf("SI_WRITE_ERROR			0x04\n");
	printf("SI_RESET_ERROR			0x05\n");
	printf("SI_INVALID_PARAMETER		0x06\n");
	printf("SI_INVALID_REQUEST_LENGTH	0x07\n");
	printf("SI_DEVICE_IO_FAILED		0x08\n");
	printf("SI_INVALID_BAUDRATE		0x09\n");
	fflush(stdout);
	closeDevice();
}

// this resets overrun flag, according to reply to post to si labs usb user forum

SI_STATUS flushBuffers(){
	SI_STATUS status;
	status = (SI_FlushBuffers)(deviceHandle,1,1); // flush both TX and RX buffers
	if(status!=SI_SUCCESS){
		printf("USBAEMonitor.c: error SI_FlushBuffers, SI_STATUS=%d\n",status);
		onError(status);
		printf("USBAEMonitor.c: error SI_FlushBuffers\n");
		fflush(stdout);
	}
//	wrapAdd=0;	// we used to reset timestamps but don't anymore, because we probably are just clearing overrun flag
	return status;
}

#ifdef __cplusplus
}
#endif
